{-# OPTIONS -fglasgow-exts #-}
-- Roshan P. James, Amr Sabry
-- rpjames@cs.indiana.edu, sabry@cs.indiana.edu

module CpsYield (MonadYield(..), Iterator(..), Yield) where 

-- This implementation of yield relies of CPS transformation of the
-- iterator. Yield is implemented as a function which returns an
-- iterator pair with the current continuation.

-- The monad relies of the following transformation - 
-- run e = cps(e) \v.(Done v)
-- cps(yield e) = \k.(cps(e) \v.(Iterator (v (\x.x)) k))
-- cps(yield v) = \k.(Iterator (v (\x.x)) k)

data Iterator i o r = Result r
		    | Susp o (i -> Iterator i o r)

newtype Yield i o r = 
    Yield {unY :: (forall b . (r -> (Iterator i o b)) -> (Iterator i o b))}

instance Monad (Yield i o) where
    return e = Yield (\k -> k e)
    (Yield e) >>= f = Yield (\k ->  e (\v -> (unY (f v)) k))

class Monad m => MonadYield i o m | m -> i, m -> o where
    yield :: o -> m i
    run :: m r -> Iterator i o r

instance MonadYield i o (Yield i o) where
    yield v = Yield (Susp v)
    run (Yield e) = e Result


